/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.core.databinding.observable.set.WritableSet;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.springframework.transaction.TransactionStatus;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.init.TermNotFoundException;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * The class controlling the plug-in life cycle.
 * </p>
 * <ul>
 * 	<li>Initializes datastore as necessary.</li>
 * 	<li>Initializes default preferences.</li>
 * 	<li>Gateway to CDM service layer.</li>
 * 	<li>Stores taxa for application session.</li>
 * 	<li>Stores registries for colors, fonts, images.</li>
 * </ul>
 * 
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class TaxEditorPlugin extends AbstractUIPlugin {
	private static final Logger logger = Logger
			.getLogger(TaxEditorPlugin.class);

	/**
	 * The plug-in ID
	 */
	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.plugin";
	/**
	 * The shared instance
	 */
	private static TaxEditorPlugin plugin;

//	private DbSchemaValidation dbSchemaValidation = DbSchemaValidation.VALIDATE;
	private DbSchemaValidation dbSchemaValidation = DbSchemaValidation.UPDATE;

	/**
	 * The constructor
	 */
	public TaxEditorPlugin() {
//		logger.fatal("Fatal");
//		logger.error("Error");
//		logger.debug("Debug");
//		logger.info("Info");
//		logger.warn("Warn");
//		logger.trace("Trace");	
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
				
		// If the preferences INITIALIZED has not been set, the IF clause
		// will return false, i.e. datastore will be initialized first
		// time the application is run after being installed.
		boolean initialized = false;
		if (getPreferenceStore()
				.contains(ITaxEditorConstants.INITIALIZED)) {
			initialized = getPreferenceStore().getBoolean(
					ITaxEditorConstants.INITIALIZED);
		}
	
		if (!initialized) {
			
			logger.warn("Initializing datastore");
			
			dbSchemaValidation = DbSchemaValidation.CREATE;
			
			if (getPreferenceStore().getBoolean(
					ITaxEditorConstants.INITIALIZE_W_TESTDATA)) {
				initDatastore();
				getPreferenceStore().setValue(
						ITaxEditorConstants.INITIALIZE_W_TESTDATA, false);				
			}

			getPreferenceStore()
					.setValue(ITaxEditorConstants.INITIALIZED, true);
		}

		cdmApp = getCdmApp();
		
		Locale locale = new Locale("en", "", "icbn");
		Locale.setDefault(locale);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		commitTransaction();
		plugin = null;
		super.stop(context);
		
		disposeColors();
	}
	
	void initDatastore() {

		boolean useSoraya = true;
		// useSoraya = false;
		Taxon genusTaxon;
		if (useSoraya) {
			genusTaxon = getSorayasGenusTaxon();
		} else {

			BotanicalName botanicalName = BotanicalName.NewInstance(Rank
					.GENUS());
			botanicalName.setTitleCache("Hieracium L.");
			botanicalName.setGenusOrUninomial("Hieracium");
			botanicalName.setCombinationAuthorTeam(Person.NewInstance());
			botanicalName.getCombinationAuthorTeam()
					.setNomenclaturalTitle("L.");
			genusTaxon = Taxon.NewInstance(botanicalName, CdmUtil
					.getSessionDefaultSec());

			BotanicalName botSpecies = BotanicalName
					.NewInstance(Rank.SPECIES());
			botSpecies.setTitleCache("Hieracium asturianum Pau");
			botSpecies.setGenusOrUninomial("Hieracium");
			botSpecies.setSpecificEpithet("asturianum");
			botSpecies.setCombinationAuthorTeam(Person.NewInstance());
			botSpecies.getCombinationAuthorTeam().setNomenclaturalTitle("Pau");
			Taxon childTaxon = Taxon.NewInstance(botSpecies, null);
			childTaxon.setTaxonomicParent(genusTaxon, null, null);

			BotanicalName botSpecies2 = BotanicalName.NewInstance(Rank
					.SPECIES());
			botSpecies2.setTitleCache("Hieracium wolffii Zahn");
			botSpecies2.setGenusOrUninomial("Hieracium");
			botSpecies2.setSpecificEpithet("wolffii");
			botSpecies2.setCombinationAuthorTeam(Person.NewInstance());
			botSpecies2.getCombinationAuthorTeam()
					.setNomenclaturalTitle("Zahn");
			Taxon childTaxon2 = Taxon.NewInstance(botSpecies2, CdmUtil
					.getSessionDefaultSec());
			// childTaxon2.setName(botSpecies2);
			// childTaxon2.setSec(null);
			childTaxon2.setTaxonomicParent(genusTaxon, null, null);

			// Add some name relations to botSpecies2
			botSpecies2.addRelationshipFromName(botSpecies,
					NameRelationshipType.BASIONYM(), null);
			botSpecies2.addRelationshipToName(botSpecies, NameRelationshipType
					.REPLACED_SYNONYM(), null);
			logger.warn("Name relations created");

			// Add name status to botSpecies2
			botSpecies2.addStatus(NomenclaturalStatus
					.NewInstance(NomenclaturalStatusType.NOVUM()));
			logger.warn("Nom. status created");
		}

		getCdmApp().getTaxonService().saveTaxon(genusTaxon);

		// System.exit(-1);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static TaxEditorPlugin getDefault() {
		return plugin;
	}

	/***************************************************************************
	 * CDM SERVICES
	 **************************************************************************/
	/**
	 * All CDM services are called via the application controller
	 */
	private CdmApplicationController cdmApp;
	private TransactionStatus txStatus;

	public CdmApplicationController getCdmApp() {
		if (cdmApp == null) {
			try {
//				cdmApp.getDatabaseService().getDatabaseEnum();
//				cdmApp.getDatabaseService().saveDataSource(strDataSourceName, databaseTypeEnum, server, database, username, password)
				
//				// Open ds by string
//				CdmPersistentDataSource ds = CdmPersistentDataSource.NewInstance(dataSourceName)
//				
//				// Open default instance
//				CdmPersistentDataSource ds = CdmPersistentDataSource.NewDefaultInstance();
//				
//				CdmPersistentDataSource.getAllDataSources();
//				
//				// Open CdmApp w default DB
//				CdmApplicationController.NewInstance(dbSchemaValidation);
				
//				System.exit(-1);
				
				cdmDatasource = 
						CdmDataSource.NewH2EmbeddedInstance("cdm", "sa", "");
////						CdmDataSource.NewMySqlInstance("192.168.2.10", "cdm_editor_test1", "edit", "wp5");
////						CdmPersistentDataSource.NewLocalHsqlInstance();
				cdmApp = CdmApplicationController
						.NewInstance(cdmDatasource, dbSchemaValidation);
 				
//				cdmApp = CdmApplicationController.NewInstance(dbSchemaValidation);

				startTransaction();
				
				// cdmApp.commitTransaction((TransactionStatus) txStatus);

			} catch (DataSourceNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TermNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return cdmApp;
	}

	public void startTransaction() {
		if (cdmApp != null) {
			txStatus = cdmApp.startTransaction();
		}
	}

	public void commitTransaction() {
		if (txStatus != null) {
			cdmApp.commitTransaction(txStatus);
			txStatus = null;
		}
	}

	private ITaxonService taxonService;

	public ITaxonService getTaxonService() {
		if (taxonService == null) {
			taxonService = getCdmApp().getTaxonService();
		}
		return taxonService;
	}
	
	private INameService nameService;

	public INameService getNameService() {
		if (nameService == null) {
			nameService = getCdmApp().getNameService();
		}
		return nameService;
	}
	
	private IDescriptionService descriptionService;
	
	public IDescriptionService getDescriptionService() {
		if (descriptionService == null) {
			descriptionService = getCdmApp().getDescriptionService();
		}
		return descriptionService;	
	}
	
	private IReferenceService referenceService; 
	
	public IReferenceService getReferenceService() {
		if (referenceService == null) {
			referenceService = getCdmApp().getReferenceService();
		}
		return referenceService;
	}

	private List<ICdmDataSource> dataSources;
	
	public List<ICdmDataSource> getDataSources() {
		if (dataSources == null) {
			dataSources = new ArrayList<ICdmDataSource>();
			dataSources.add(CdmDataSource.
					NewH2EmbeddedInstance("cdm", "sa", ""));
			dataSources.add(CdmDataSource.
					NewMySqlInstance("192.168.2.10", "cdm_editor_test1", 
							"edit", "wp5"));
		}
		return dataSources;
	}
	
	private ICdmDataSource cdmDatasource;	
	
	public ICdmDataSource getCdmDataSource() {
		return cdmDatasource;
	}

	/***************************************************************************
	 * OBSERVABLE LISTS
	 **************************************************************************/

	private WritableList observableRecentNamesList = null;
	private WritableList observableFavoritesList = null;

	public WritableList getObservableRecentNamesList() {
		if (observableRecentNamesList == null) {
			observableRecentNamesList = new WritableList();
		}
		return observableRecentNamesList;
	}

	public WritableList getObservableFavoritesList() {
		if (observableFavoritesList == null) {
			observableFavoritesList = new WritableList();
		}
		return observableFavoritesList;
	}

	/***************************************************************************
	 * IMAGE REGISTRY
	 **************************************************************************/
	public ImageDescriptor getImageDescriptor(String key) {
		return getImageRegistry().getDescriptor(key);
	}

	public Image getImage(String key) {
		return getImageRegistry().get(key);
	}

	// Resource bundle.
	private ResourceBundle resourceBundle;
	private FormColors formColors;

	protected void initializeImageRegistry(ImageRegistry registry) {
		registerImage(registry, ITaxEditorConstants.EDIT_ICON, "edit_16x16.ico");
		registerImage(registry, ITaxEditorConstants.WARNING_ICON,
				"warn_tsk.gif");
		registerImage(registry, ITaxEditorConstants.BLACK_SQUARE_ICON,
				"accepted_small.gif");
		registerImage(registry, ITaxEditorConstants.HOMOTYPIC_SYN_ICON,
				"homosyn_no_bg.gif");
		registerImage(registry,
				ITaxEditorConstants.HOMOTYPIC_SYN_ORIGINAL_ICON,
				"homosyn_original_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.HETEROTYPIC_SYN_ICON,
				"heterosyn_no_bg.gif");
		registerImage(registry,
				ITaxEditorConstants.HETEROTYPIC_SYN_ORIGINAL_ICON,
				"heterosyn_original_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.MISAPPLIED_NAME_ICON,
				"misapplied_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.AUTONYM_ICON,
				"autonym_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.BASIONYM_ICON,
				"basionym_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.ORTHOGRAPHIC_VARIANT_ICON,
				"orthovariant_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.DB_ICON, "db.gif");
		registerImage(registry, ITaxEditorConstants.MOVE_ICON,
				"correction_change.gif");
		registerImage(registry, ITaxEditorConstants.ACTIVE_DELETE_ICON,
				"delete_edit.gif");
		registerImage(registry, ITaxEditorConstants.SYNONYM_TO_TAXON_ICON,
				"change.gif");
		registerImage(registry, ITaxEditorConstants.OPEN_TAXON_ICON, "open.gif");
		registerImage(registry, ITaxEditorConstants.ADD_CHILD_TAXON_ICON,
				"new_child.gif");
		registerImage(registry,
				ITaxEditorConstants.SWAP_SYNONYM_AND_TAXON_ICON, "swap2.gif");
		registerImage(registry, ITaxEditorConstants.QUICK_ADD_ICON,
				"quick_add.gif");
		registerImage(registry, ITaxEditorConstants.TAXON_TO_SYNONYM_ICON,
				"tax_to_syn.gif");
	}

	private void registerImage(ImageRegistry registry, String key,
			String fileName) {
		try {
			IPath path = new Path("icons/" + fileName); //$NON-NLS-1$
			URL url = FileLocator.find(getBundle(), path, null);
			if (url != null) {
				ImageDescriptor desc = ImageDescriptor.createFromURL(url);
				registry.put(key, desc);
			}
		} catch (Exception e) {
		}
	}

	/***************************************************************************
	 * FONT REGISTRY
	 **************************************************************************/
	private FontRegistry fontRegistry;

	private FontRegistry getFontRegistry() {
		if (fontRegistry == null) {
			fontRegistry = new FontRegistry(Display.getCurrent());

			fontRegistry.put(ITaxEditorConstants.DATASOURCE_FONT,
					new FontData[] { new FontData("Arial", 8, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.MENU_ITEM_ITALICS_FONT,
					new FontData[] { new FontData("Arial", 9, SWT.ITALIC) });
			fontRegistry.put(ITaxEditorConstants.ACCEPTED_TAXON_FONT,
					new FontData[] { new FontData("Georgia", 12, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.SYNONYM_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.MISAPPLIEDNAME_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.CHOOSE_NAME_TEXT_FONT,
					new FontData[] { new FontData("Arial", 12, SWT.BOLD) });
			fontRegistry.put(ITaxEditorConstants.DEFAULT_PROMPT_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.ITALIC) });
		}
		return fontRegistry;
	}

	public Font getFont(String key) {
		return getFontRegistry().get(key);
	}

	/***************************************************************************
	 * COLOR MAP
	 **************************************************************************/
	private static HashMap<String, Color> colorMap;
	
	public Color getColor(String key) {
		return getColorMap().get(key);
	}
	
	private HashMap<String, Color> getColorMap() {
		if (colorMap == null) {
			 colorMap = new HashMap<String, Color>();
			 
			 colorMap.put(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR,
					 new Color(null, 240, 240, 240));
		}
		return colorMap;
	}
	
	private void disposeColors() {
		if (colorMap == null) {
			return;
		}
        for (Color color : colorMap.values()) {
        	color.dispose();
        }
        colorMap.clear();
	}

	/***************************************************************************
	 * TAXONOMIC TREE
	 **************************************************************************/

	public ReferenceBase getSec() {
		return getCdmApp().getReferenceService().getReferenceByUuid(
				UUID.fromString("f3593c18-a8d2-4e51-bdad-0befbf8fb2d1"));
	}

	private Set<Taxon> sessionRootTaxa;
	/**
	 * The taxonomic tree content provider observes this set for changes, and
	 * makes the appropriate changes to the tree.
	 */
	private IObservableSet observableSessionTaxonSet;
	/**
	 * This map of taxa to their taxonomic children is used by
	 * NameTreeContentProvider's getChildren method.
	 * 
	 * key = taxon, value = key's child taxa Note that a map can have a key ==
	 * null, i.e. no parent taxon
	 */
	private HashMap<Taxon, Set<Taxon>> sessionTaxonomicChildrenMap;

	public Set<Taxon> getSessionRootTaxa() {
		if (sessionRootTaxa == null) {
			
			sessionRootTaxa = new HashSet<Taxon>();
//			sessionRootTaxa.addAll(getCdmApp().getTaxonService().getRootTaxa(
//					getSec(), null, false));
			sessionRootTaxa.addAll(getCdmApp().getTaxonService().getRootTaxa(
					getSec(), null, true));			
			addSessionTaxa(sessionRootTaxa);
						
		}
		return sessionRootTaxa;
	}

	private Map<Taxon, Set<Taxon>> getTaxonomicChildrenMap() {
		if (sessionTaxonomicChildrenMap == null) {
			sessionTaxonomicChildrenMap = new HashMap<Taxon, Set<Taxon>>();
		}
		return sessionTaxonomicChildrenMap;
	}

	/**
	 * Returns a set of any child taxa for this taxon that have already been
	 * requested during this session.
	 * 
	 * @param parentTaxon
	 * @return
	 */
	public Set<Taxon> getSessionTaxonomicChildren(Taxon parentTaxon) {
		if (!getTaxonomicChildrenMap().containsKey(parentTaxon)) {
			if (parentTaxon == null) {
				getTaxonomicChildrenMap().put(parentTaxon, getSessionRootTaxa());
			} else {
			
				// BUG deleted taxon were re-appearing as NULL children
				Set<Taxon> childTaxa = new HashSet<Taxon>();
				
//				TransactionStatus tx = getCdmApp().startTransaction();
//				getCdmApp().getTaxonService().saveTaxon(parentTaxon);
				for (Taxon taxon : parentTaxon.getTaxonomicChildren()) {
					if (taxon != null) {
						childTaxa.add(taxon);
					}
				}
//				getCdmApp().commitTransaction(tx);
				
				getTaxonomicChildrenMap().put(parentTaxon, childTaxa);
			}
		}
		return getTaxonomicChildrenMap().get(parentTaxon);
	}

	/**
	 * @return
	 */
	public IObservableSet getObservableSessionTaxa() {
		if (observableSessionTaxonSet == null) {
			Realm realm = SWTObservables.getRealm(Display.getDefault());
			observableSessionTaxonSet = new WritableSet(realm);
		}
		return observableSessionTaxonSet;
	}

	/**
	 * Recursive function to clear the hashmap of a taxon's children used in
	 * this session, and the childrens' children, etc.
	 * 
	 * @param taxon
	 */
	private void clearTaxonomicChildren(Taxon taxon) {
		Set<Taxon> children = getSessionTaxonomicChildren(taxon);
		if (children != null) {
			for (Taxon child : children) {
				clearTaxonomicChildren(child);
			}
			getTaxonomicChildrenMap().remove(taxon);
			getObservableSessionTaxa().removeAll(children);
		}
	}

	/**
	 * Remove taxon from all session collections
	 * 
	 * @param taxon
	 */
	public void removeSessionTaxon(Taxon taxon) {

		// Recursively remove all children, children's children, etc.
		clearTaxonomicChildren(taxon);

		Taxon parentTaxon = taxon.getTaxonomicParent();

		// Remove from parent's child map, or from root taxa
		if (parentTaxon == null) {
			getSessionRootTaxa().remove(taxon);
		} else {
			getSessionTaxonomicChildren(parentTaxon).remove(taxon);
		}

		// Remove from session taxa
		getObservableSessionTaxa().remove(taxon);
		
		UiUtil.getTreeViewer().remove(taxon);
	}

	public void addSessionTaxa(Collection<Taxon> taxa) {
		for (Taxon taxon : taxa) {
			addSessionTaxon(taxon);
		}
	}

	/**
	 * Whenever a taxon is opened, either for editing or for display in the
	 * taxonomic tree, it should be added to the collections of taxa used during
	 * this session.
	 * 
	 * @param taxon
	 */
	public void addSessionTaxon(Taxon taxon) {

		// Add taxon to session taxa if not already there
		if (!(getObservableSessionTaxa().contains(taxon))) {
			getObservableSessionTaxa().add(taxon);
		}
		
		Taxon parentTaxon = taxon.getTaxonomicParent();

		// If taxon has no parent, add it to root taxa
		if (parentTaxon == null) {
			if (!(getSessionRootTaxa().contains(taxon))) {
				getSessionRootTaxa().add(taxon);
			}
		}

		// Add taxon to its parent's child map
		getSessionTaxonomicChildren(parentTaxon).add(taxon);
				
		TaxonomicTreeViewer viewer = UiUtil.getTreeViewer();
		
		if (viewer != null) {
			
			if (parentTaxon == null) {
				viewer.setInput(getSessionRootTaxa());
			} else {
				viewer.add(parentTaxon, taxon);			
			}
		
		}
	}

	
//	private TaxonomicTreeView taxonomicTreeView;
//	
//	public void setTaxonomicTree(TaxonomicTreeView taxonomicTreeView) {
//		this.taxonomicTreeView = taxonomicTreeView;
//	}
	
	/***************************************************************************
	 * RANKS
	 **************************************************************************/
	OrderedTermVocabulary<Rank> rankVocabulary = null;

	public OrderedTermVocabulary<Rank> getRankVocabulary() {
		if (rankVocabulary == null) {
			rankVocabulary = getCdmApp().getNameService().getRankVocabulary();
		}
		return rankVocabulary;
	}

	/***************************************************************************
	 * NOMENCLATURAL STATUS
	 **************************************************************************/
	TermVocabulary<NomenclaturalStatusType> nomStatusVocabulary = null;

	public TermVocabulary<NomenclaturalStatusType> getNomStatusVocabulary() {
		if (nomStatusVocabulary == null) {
			nomStatusVocabulary = getCdmApp().getNameService()
					.getStatusTypeVocabulary();
		}
		return nomStatusVocabulary;
	}

	/***************************************************************************
	 * NAME RELATIONS
	 **************************************************************************/
	TermVocabulary<NameRelationshipType> nameRelationshipTypeVocabulary = null;

	public TermVocabulary<NameRelationshipType> getNameRelationshipTypeVocabulary() {
		if (nameRelationshipTypeVocabulary == null) {
			nameRelationshipTypeVocabulary = getCdmApp().getNameService()
					.getNameRelationshipTypeVocabulary();
		}
		return nameRelationshipTypeVocabulary;
	}

	public Set<NameRelationshipType> getSortedNameRelationshipTypes() {
		return getNameRelationshipTypeVocabulary().getTermsOrderedByLabels(
				Language.DEFAULT());
	}

	/***************************************************************************
	 * SORAYA DATA
	 **************************************************************************/
	private Taxon getSorayasGenusTaxon() {

		String[] children = new String[] {
				"Heterospathe annectens H.E.Moore",
				"Heterospathe arfakiana (Becc.) H.E.Moore",
				"Heterospathe brevicaulis Fernando",
				"Heterospathe cagayanensis Becc.",
				"Heterospathe califrons Fernando, Palms 45: 118 (2001).",
				"Heterospathe clemensiae (Burret) H.E.Moore",
				"Heterospathe delicatula H.E.Moore",
				"Heterospathe dransfieldii Fernando",
				"Heterospathe elata Scheff.",
				"Heterospathe elata var. elata.",
				"Heterospathe elata var. guamensis Becc.",
				"Heterospathe elata var. palauensis (Becc.) Becc.",
				"Heterospathe elegans (Becc.) Becc.",
				"Heterospathe elmeri Becc.",
				"Heterospathe glabra (Burret) H.E.Moore",
				"Heterospathe glauca (Scheff.) H.E.Moore",
				"Heterospathe humilis Becc.",
				"Heterospathe intermedia (Becc.) Fernando",
				"Heterospathe kajewskii Burret",
				"Heterospathe ledermanniana Becc.",
				"Heterospathe lepidota H.E.Moore",
				"Heterospathe longipes (H.E.Moore) Norup",
				"Heterospathe macgregorii (Becc.) H.E.Moore",
				"Heterospathe micrantha (Becc.) H.E.Moore",
				"Heterospathe minor Burret",
				"Heterospathe muelleriana (Becc.) Becc.",
				"Heterospathe negrosensis Becc.",
				"Heterospathe obriensis (Becc.) H.E.Moore",
				"Heterospathe palauensis Becc.",
				"Heterospathe parviflora Essig",
				"Heterospathe philippinensis (Becc.) Becc.",
				// "Heterospathe phillipsii D.Fuller & Dowe", // <--------------
				"Heterospathe pilosa (Burret) Burret",
				"Heterospathe pisifera (Gaertn.) Burret",
				"Heterospathe pulchra H.E.Moore",
				"Heterospathe ramulosa Burret",
				"Heterospathe salomonensis Becc.",
				"Heterospathe scitula Fernando", "Heterospathe sensisi Becc.",
				"Heterospathe sibuyanensis Becc.",
				"Heterospathe sphaerocarpa Burret",
				"Heterospathe trispatha Fernando",
				"Heterospathe uniformis Dowe",
				"Heterospathe versteegiana Becc.",
				"Heterospathe woodfordiana Becc." };

		BotanicalName genusName = BotanicalName
				.PARSED_NAME("Heterospathe Scheff.");

		Taxon genusTaxon = Taxon.NewInstance(genusName, CdmUtil
				.getSessionDefaultSec());

		for (String child : children) {
			BotanicalName speciesName = BotanicalName.PARSED_NAME(child);

			Taxon childTaxon = Taxon.NewInstance(speciesName, CdmUtil
					.getSessionDefaultSec());
			childTaxon.setTaxonomicParent(genusTaxon, null, null);

			if (child.equals("Heterospathe elegans (Becc.) Becc.")) {
				BotanicalName basionym = BotanicalName
						.PARSED_NAME("Barkerwebbia elegans Becc.");
				speciesName.addRelationshipFromName(basionym,
						NameRelationshipType.BASIONYM(), null);
				childTaxon.addHomotypicSynonymName(basionym, null, null);
			}

			if (child.equals("Heterospathe humilis Becc.")) {
				BotanicalName synonymName = BotanicalName
						.PARSED_NAME("Barkerwebbia humilis (Becc.) Becc. ex Martelli");
				childTaxon.addHomotypicSynonymName(synonymName, null, null);
			}
		}

		return genusTaxon;
	}
}