/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.operations.IWorkbenchOperationSupport;
import org.eclipse.ui.views.properties.PropertySheet;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.CompositeBorderDecorator;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.model.NameEditorInput;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * A collection of useful methods related to the UI.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class UiUtil {
	private static final Logger logger = Logger.getLogger(UiUtil.class);

	private static Set<Feature> preferredFeatureSet;
	private static Set<Rank> preferredRankSet;
	private static IViewPart treeView;
	
	public static IViewPart getPropertySheet() {
		for (IViewReference reference : getActivePage().getViewReferences()) {
			if (reference.getId().equals(IPageLayout.ID_PROP_SHEET)) {
				return reference.getView(false);
			}
		}
		return null;
	}
	
	/**
	 * By default, property sheet has buttons in the toolbar for 
	 * "Show advanced properties" and "Show categories".
	 * <p>
	 * This is confusing for the user, hence a method to remove them
	 * until such time as advanced properties or categories are implemented.
	 */
	public static void hidePropertySheetToolbar() {
		PropertySheet propertySheet = (PropertySheet) getPropertySheet();
		IActionBars actionBars = propertySheet.getViewSite().getActionBars();
		actionBars.getToolBarManager().removeAll();
		actionBars.getMenuManager().removeAll();
	}
	
	/**
	 * The property sheet listener ensures only property sheets
	 * 	with data cause the Property Sheet to be updated.
	 */
	public static void addPropertySheetInputListener() {
		IViewPart propertySheet = getPropertySheet();
//		propertySheet.get
		PropertySheet ps = (PropertySheet) propertySheet;
//		ps.addPartPropertyListener(listener)
//		ps.addPropertyListener(l)
	}
	
	/**
	 * @param input
	 * @return
	 * @throws PartInitException
	 */
	public static IEditorPart getEditorByInput(IEditorInput input)
			throws PartInitException {
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			if (reference.getEditorInput().equals(input)) {
				IEditorPart editor = reference.getEditor(false);
				return editor;
			}
		}
		return null;
	}
	
	/**
	 * Returns a set of all currently open 
	 * <code>MultiPageTaxonEditor</code>s.
	 * 
	 * @return
	 */
	public static Set<IEditorPart> getOpenTaxonEditors() {
		
		Set<IEditorPart> taxonEditors = new HashSet<IEditorPart>();
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			IEditorPart editor = reference.getEditor(false);
			if (editor instanceof MultiPageTaxonEditor) {
				taxonEditors.add(editor);
			}
		}
		return taxonEditors;
	}

	public static IEditorPart getEditorByTaxon(Taxon taxon)
			throws PartInitException {
		IEditorInput input = new NameEditorInput(taxon);
		return getEditorByInput(input);
	}
	
	/**
	 * @return
	 */
	public static IWorkbenchPage getActivePage() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
	}

	/**
	 * @return
	 */
	public static Shell getShell() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}

	/**
	 * @param input
	 * @param editorId
	 * @throws PartInitException
	 */
	public static void openEditor(IEditorInput input, String editorId)
			throws PartInitException {
		getActivePage().openEditor(input, editorId);
	}

	/**
	 * @param taxon
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(Taxon taxon, boolean save)
			throws PartInitException {
		IEditorPart editor = getEditorByTaxon(taxon);
		closeEditor(editor, save);
	}
	
	/**
	 * @param input
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(IEditorInput input, boolean save)
			throws PartInitException {
		IEditorPart editor = getEditorByInput(input);
		closeEditor(editor, save);
	}

	/**
	 * @param editor
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(IEditorPart editor, boolean save)
			throws PartInitException {
		if (editor != null) {
			getActivePage().closeEditor(editor, save);
		}
	}

	public static IViewPart getViewById(String id) {
		return TaxEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
				getActivePage().findView(id);
	}
	

	/**
	 * Returns the <code>TaxonomicTreeViewer</code> used for navigation. Not to 
	 * be confused with the window that contains it, <code>TaxonomicTreeView</code>. 
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer getTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.getTreeViewer();
	}

	/**
	 * @see eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView#createTreeViewer()
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer createTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.createTreeViewer();
	}
	
	/**
	 * Returns the <code>TaxonomicTreeView</code> containing the 
	 * <code>TaxonomicTreeViewer</code> used for navigation. 
	 * 
	 * @return
	 */	
	public static TaxonomicTreeView getTreeView() {
		if (treeView == null) {
			treeView = getViewById(TaxonomicTreeView.ID);
			if (!(treeView instanceof TaxonomicTreeView)) {
				treeView = null;
			}
		}
		return (TaxonomicTreeView) treeView;
	}
	

	public static IWorkbenchOperationSupport getOperationSupport() {
		IWorkbench workbench = TaxEditorPlugin.getDefault().getWorkbench();
		return workbench.getOperationSupport();
	}
	
	public static IOperationHistory getOperationHistory() {
		return OperationHistoryFactory.getOperationHistory();
//		return getOperationSupport().getOperationHistory();
	}

	public static IUndoContext getWorkbenchUndoContext() {
		return getOperationSupport().getUndoContext();
	}

	public static void doEditorSave(Taxon taxon, boolean confirm) {
		IEditorPart editor = null;
		try {
			editor = getEditorByTaxon(taxon);
		} catch (PartInitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (editor != null) {
			getActivePage().saveEditor(editor, confirm);
		}
	}
	

	public static EditorGroupComposite getMisappliedNameGroupComposite(
			IManagedForm managedForm) {

		// Iterate through parent's children until we find a composite which has a data field
		//	for MISAPPLIED_NAME
		Composite parent = managedForm.getForm().getBody();
		for (Control groupComposite : parent.getChildren()) {
			if (groupComposite instanceof EditorGroupComposite) {
				if (groupComposite.getData(ITaxEditorConstants.MISAPPLIED_NAME) != null) {
					return (EditorGroupComposite) groupComposite;
				}
			}
		}
		return null;
	}

	public static EditorGroupComposite createMisappliedNameGroupComposite(
			IManagedForm managedForm) {
		
		Composite parent = managedForm.getForm().getBody();
		
		EditorGroupComposite composite = new EditorGroupComposite(parent, managedForm);
		composite.setData(ITaxEditorConstants.MISAPPLIED_NAME,"");
		new CompositeBorderDecorator(composite, managedForm);
		return composite;
	}
	
	public static IPreferenceStore getPrefStore() {
		return TaxEditorPlugin.getDefault().getPreferenceStore();
	}
	
	/**
	 * Returns a <code>Set</code> of the <code>Feature</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Feature</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static Set<Feature> getPreferredFeatures() {
		
		// Initialize preferredFeatureSet as necessary
		if (preferredFeatureSet == null) {
			
			preferredFeatureSet = new HashSet<Feature>();	
						
			for (Feature feature : CdmUtil.getFeatures()) {
				
				// If the feature is set to show, add it to preferredFeatureSet
				if (getFeaturePreference(feature)) {
					preferredFeatureSet.add(feature);
				}
			}
			
		}
		return preferredFeatureSet;
	}
	
	/**
	 * True if <code>Feature</code> is set to "show" in preferences.
	 * 
	 * @param feature
	 * @return
	 */
	public static boolean getFeaturePreference(Feature feature) {
		
		String preferenceKey = getPreferenceKey(feature);
		
		// If feature does not yet have a pref, set it to true
		if (!getPrefStore().contains(preferenceKey)) {
			getPrefStore().setDefault(preferenceKey, true);
		}
		
		return getPrefStore().getBoolean(preferenceKey);
	}
	
	/**
	 * Set the show state of a <code>Feature</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredFeatureSet</code> to null to force it be
	 * re-populated the next time {@link #getPreferredFeatures()} is called.
	 * 
	 * @param feature
	 * @param show
	 */
	public static void setFeaturePreference(Feature feature, boolean show) {
		preferredFeatureSet = null;
		getPrefStore().setValue(getPreferenceKey(feature), show);
	}
	
	/**
	 * Construct a unique key using <code>Feature</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Feature feature) {
		return ITaxEditorConstants.FEATURE_PREFERENCE 
				. concat(".")
				. concat(feature.getUuid().toString());
	}

	/**
	 * Returns a <code>Set</code> of the <code>Rank</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Rank</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static Set<Rank> getPreferredRanks() {
		
		// Initialize preferredFeatureSet as necessary
		if (preferredRankSet == null) {
			
			preferredRankSet = new HashSet<Rank>();	
						
			for (Rank rank : CdmUtil.getRanks()) {
				
				// If the feature is set to show, add it to preferredFeatureSet
				if (getRankPreference(rank)) {
					preferredRankSet.add(rank);
				}
			}
			
		}
		return preferredRankSet;
	}	
	
	/**
	 * True if <code>Rank</code> is set to "show" in preferences.
	 * 
	 * @param rank
	 * @return
	 */
	public static boolean getRankPreference(Rank rank) {
		
		String preferenceKey = getPreferenceKey(rank);
		
		// If rank does not yet have a pref, set it to true
		if (!getPrefStore().contains(preferenceKey)) {
			getPrefStore().setDefault(preferenceKey, true);
		}
		
		return getPrefStore().getBoolean(preferenceKey);
	}
	
	/**
	 * Set the show state of a <code>Rank</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredRankSet</code> to null to force it be
	 * re-populated the next time {@link #getPreferredRanks()} is called.
	 * 
	 * @param rank
	 * @param show
	 */	
	public static void setRankPreference(Rank rank, boolean show) {
		preferredRankSet = null;
		getPrefStore().setValue(getPreferenceKey(rank), show);
		
	}
	
	/**
	 * Construct a unique key using <code>Rank</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Rank rank) {
		return ITaxEditorConstants.RANK_PREFERENCE 
				. concat(".")
				. concat(rank.getUuid().toString());
	}	
	
	/**
	 * @return
	 */
	public static NonViralName getPreferredNameClassInstance() {
		String nameCodePreference = TaxEditorPlugin.getDefault().getPreferenceStore().getString(ITaxEditorConstants.CODE_PREFERENCE);
		
		// Check whether name code preference needs to be initialized
		if (nameCodePreference == null || nameCodePreference.equals("")) {
			nameCodePreference = ITaxEditorConstants.DEFAULT_CODE_PREFERENCE;
			UiUtil.getPrefStore().setValue(ITaxEditorConstants.CODE_PREFERENCE, 
					nameCodePreference);
		}
		
		if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICBN)) {
			return BotanicalName.NewInstance(null);
		} else if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			return ZoologicalName.NewInstance(null);			
		}
		return NonViralName.NewInstance(null);
	}
	
	public static String getNameCodePreference() {
		String nameCodePreference = getPrefStore().getString(ITaxEditorConstants.CODE_PREFERENCE);
		
		// Check whether name code preference needs to be initialized
		if (nameCodePreference == null || nameCodePreference.equals("")) {
			nameCodePreference = ITaxEditorConstants.DEFAULT_CODE_PREFERENCE;
			getPrefStore().setValue(ITaxEditorConstants.CODE_PREFERENCE, nameCodePreference);
		}
		
		return nameCodePreference;
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {

		String nameCodePreference = getNameCodePreference();
				
		if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICBN)) {
			return NomenclaturalCode.ICBN();
		} else if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			return NomenclaturalCode.ICZN();
		}
		return null;
	}

	/**
	 * Appends the code (i.e. ICBN or ICZN) specified in the user preferences
	 * to a string. The result is separated by a period: "<code>key.ICBN</code>".
	 * 
	 * @param key
	 * @return
	 */
	public static String concatCodeMessageSuffix(String key) {
		String code = getNameCodePreference();
		return key + "." + code;
	}
	
	public static void setStatusMessage(String msg) {
		getPropertySheet().getViewSite().getActionBars().getStatusLineManager().setMessage(msg); 
	}
}