/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Calls <code>MultiPageTaxonEditor.doSave()</code> method for all
 * open <code>MultiPageTaxonEditor</code>s. Commits universal transaction,
 * then opens a new one.
 * 
 * @see IWorkbenchWindowActionDelegate
 * 
 * @author p.ciardelli
 * @created 07.10.2008
 * @version 1.0
 */
public class SaveAllAction implements IWorkbenchWindowActionDelegate {
	private static final Logger logger = Logger.getLogger(SaveAllAction.class);

	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public SaveAllAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		
		// Get all open windows
		for (IEditorPart taxonEditor : UiUtil.getOpenTaxonEditors()) {

			// Save the dirty ones
			if (taxonEditor.isDirty()) {
				IEditorInput input = taxonEditor.getEditorInput();
				if (input.getAdapter(Taxon.class) != null) {
					Taxon taxon = (Taxon) input.getAdapter(Taxon.class);
					new SaveTaxonAction(taxon).run();
					if (taxonEditor instanceof MultiPageTaxonEditor) {
						((MultiPageTaxonEditor) taxonEditor).setDirtyExtern(false);
					}
				} 
				
				taxonEditor.doSave(null);
			}
		}

		// Commit the transaction
		TaxEditorPlugin.getDefault().commitTransaction();	
	
		// Force library objects to be associated with new transaction
		CdmUtil.clearLibraryObjects();
		
		// Start a new transaction
		TaxEditorPlugin.getDefault().startTransaction();
		
		// Put all open taxa in the new transaction
		for (Object taxon : TaxEditorPlugin.getDefault().getObservableSessionTaxa()) {
			CdmUtil.getTaxonService().saveTaxon((Taxon) taxon);
		}
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}