/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.AddHeterotypicSynonymToTaxonAction;
import eu.etaxonomy.taxeditor.editor.CompositeBorderDecorator;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;

/**
 * Creates a new homotypic group with one new heterotypic synonym, then creates
 * the corresponding UI elements. The synonym is created with text which is 
 * subsequently parsed.
 * 
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class CreateNewHeterotypicCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(CreateNewHeterotypicCompositeAction.class);

	private static String text = "Create a heterotypic synonym in a new homotypic group";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.HETEROTYPIC_SYN_ICON);

	private IUndoableOperation operation;

	private Taxon taxon;
	private String synonymText;
	private IManagedForm managedForm;
	
	private CreateNewHeterotypicCompositeAction() {
		super(text);
		setImageDescriptor(image);
	}

	public CreateNewHeterotypicCompositeAction(String synonymText, IManagedForm managedForm) {
		this();

		this.synonymText = synonymText;
		this.managedForm = managedForm;
		
		if (managedForm.getForm().getBody().getData() instanceof Taxon) {
			this.taxon = (Taxon) managedForm.getForm().getBody().getData();			
		} else {
			throw new IllegalArgumentException(
					"The IManagedForm's body does not have a Taxon in its data field as required.");
		}

		operation = new CreateCompositesAction();
	}
	
	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class CreateCompositesAction extends AbstractOperation {

		public CreateCompositesAction() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			
			Action action = 
				new AddHeterotypicSynonymToTaxonAction(synonymText, taxon);
			
			action.addPropertyChangeListener(new IPropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent event) {
					if (event.getProperty()
							.equals(ITaxEditorConstants.SYNONYM)) {
						if (event.getNewValue() instanceof Synonym) {
							
							// Get synonym from property change event
							Synonym synonym = (Synonym) event.getNewValue();
							
							// Get homotypic group's parent composite 
							Composite grandParent = managedForm.getForm().getBody();
														
							// Create new homotypic group composite
							Object data = synonym.getName().getHomotypicalGroup();
							EditorGroupComposite newGroupComposite = new EditorGroupComposite(grandParent, managedForm, data);
							
							// Put new group composite above misapplied names as necessary
							EditorGroupComposite misappliedNameGroupComposite = UiUtil
									.getMisappliedNameGroupComposite(managedForm);
							if (misappliedNameGroupComposite != null) {
								newGroupComposite.moveAbove(misappliedNameGroupComposite);
							}
							
							// Create new heterotypic synonym composite, init w unparsed string
							NameComposite newNameComposite = new NameComposite(newGroupComposite, managedForm, NameComposite.HETEROTYPIC_SYNONYM, synonym);
							newNameComposite.setText(synonymText);
							
							grandParent.layout();
						}
					}
				}
			});
			action.run();

			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}
	}
}
