/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui.description;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.description.TextFormat;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.editor.description.DescriptionElementComposite;
import eu.etaxonomy.taxeditor.editor.description.FeatureGroupComposite;

/**
 * Checks whether the <code>TaxonDescriptionComposite</code> contains a
 * a <code>FeatureGroupComposite</code> for the <code>Feature</code>; creates a
 * new <code>FeatureGroupComposite</code> if necessary. Then adds a new 
 * <code>DescriptionElementComposite</code> to the <code>FeatureGroupComposite</code>.
 * 
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class AddDescriptionElementCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddDescriptionElementCompositeAction.class);

	private String text;
	private ImageDescriptor image = null;

	private IUndoableOperation operation;

	private TaxonDescription taxonDescription;
	private Composite taxonDescriptionComposite;
	private IManagedForm managedForm;
	private Feature feature;

	/**
	 * descriptionElement and descriptionElementComposite are created when
	 * the action is executed, and disposed as part of any undo.
	 */
	private DescriptionElementBase descriptionElement;
	private DescriptionElementComposite descriptionElementComposite;
	
	private AddDescriptionElementCompositeAction(String text) {
		super(text);
		this.text = text;
		if (image != null) {
			setImageDescriptor(image);
		}
	}

	public AddDescriptionElementCompositeAction(Composite taxonDescriptionComposite,
			IManagedForm managedForm, Feature feature) {
		
		// The feature's label will be shown in the menu entry 
		this(feature.getLabel());

		// Make sure TaxonDescriptionComposite has a TaxonDescription in its data field
		if (taxonDescriptionComposite.getData() instanceof TaxonDescription) {
			this.taxonDescription = (TaxonDescription) taxonDescriptionComposite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a composite with a TaxonDescription in its data field.");
		}

		this.taxonDescriptionComposite = taxonDescriptionComposite;
		this.managedForm = managedForm;
		this.feature = feature;
		
		operation = new AddDescriptionElementOperation();
		
	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class AddDescriptionElementOperation extends AbstractOperation {

		public AddDescriptionElementOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
						
			// Look for a FeatureGroupComposite containing the current Feature
			FeatureGroupComposite featureGroupComposite = null;
			for (Control child : taxonDescriptionComposite.getChildren()) {
				if (child instanceof FeatureGroupComposite && child.getData() != null
						&& child.getData().equals(feature)) {
					featureGroupComposite = (FeatureGroupComposite) child;
					break;
				}
			}
			
			// Otherwise, create a new FeatureGroupComposite
			if (featureGroupComposite == null) {
				featureGroupComposite = new FeatureGroupComposite(taxonDescriptionComposite, 
						managedForm, feature);
			}
			
			// Create new element, set it as the data for a new element composite
			descriptionElement = TextData.NewInstance("", Language.DEFAULT(), null);
			descriptionElement.setFeature(feature);
			taxonDescription.addElement(descriptionElement);
			descriptionElementComposite = new DescriptionElementComposite(
					featureGroupComposite, managedForm, descriptionElement);
			
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}
	}
}
