/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.ui.AdaptCompositeToGroupAction;
import eu.etaxonomy.taxeditor.actions.ui.AddBasionymCompositeAction;
import eu.etaxonomy.taxeditor.actions.ui.ChangeCompositeToMisappliedNameAction;
import eu.etaxonomy.taxeditor.actions.ui.ChangeCompositeToNewTaxonAction;
import eu.etaxonomy.taxeditor.actions.ui.ChangeSynonymToTaxonUiAction;
import eu.etaxonomy.taxeditor.actions.ui.ChangeTaxonToSynonymAction;
import eu.etaxonomy.taxeditor.actions.ui.CreateNewHeterotypicCompositeAction;
import eu.etaxonomy.taxeditor.actions.ui.DeleteMisappliedNameCompositeAction;
import eu.etaxonomy.taxeditor.actions.ui.DeleteSynonymCompositeAction;
import eu.etaxonomy.taxeditor.actions.ui.MoveCompositeToMisappliedCompositeAction;
import eu.etaxonomy.taxeditor.actions.ui.MoveTaxonDialogAction;
import eu.etaxonomy.taxeditor.actions.ui.RemoveBasionymCompositeAction;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.EditorGroupedComposite;
import eu.etaxonomy.taxeditor.editor.LineBreakListener;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Formats an <code>EditorGroupedComposite</code> to display <code>TaxonNameBase</code> elements
 * in a <code>NameViewer</code>.
 * 
 * <code>Composite.getData()</code> returns <code>TaxonBase</code>. Therefore, the method   
 * <code>setSelection()</code> is overriden to send the property sheet <code>TaxonBase.getName()</code>. 
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class NameComposite extends EditorGroupedComposite {
	private static final Logger logger = Logger.getLogger(NameComposite.class);

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public String compositeType;
	public static final String ACCEPTED_TAXON = "accepted_name_composite";
	public static final String HOMOTYPIC_SYNONYM = "homotypic_name_composite";
	public static final String HETEROTYPIC_SYNONYM = "heterotypic_name_composite";
	public static final String MISAPPLIED_NAME = "misappliedname_name_composite";

	/**
	 * ************ INDENTATIONS ************
	 */
	public static final int ACCEPTED_INDENT = 0;
	public static final int SYNONYM_INDENT = 15;
	public static final int MISAPPLIEDNAME_INDENT = 15;

	/**
	 * ************ FONTS ************
	 */
	public static final Font ACCEPTED_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.ACCEPTED_TAXON_FONT);
	public static final Font SYNONYM_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.SYNONYM_FONT);
	public static final Font MISAPPLIEDNAME_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.MISAPPLIEDNAME_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image ACCEPTED_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.HOMOTYPIC_SYN_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = TaxEditorPlugin
			.getDefault().getImage(
					ITaxEditorConstants.HOMOTYPIC_SYN_ORIGINAL_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.HETEROTYPIC_SYN_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = TaxEditorPlugin
			.getDefault().getImage(
					ITaxEditorConstants.HETEROTYPIC_SYN_ORIGINAL_ICON);
	public static final Image MISAPPLIEDNAME_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.MISAPPLIED_NAME_ICON);
	public static final Image AUTONYM_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.AUTONYM_ICON);
	public static final Image BASIONYM_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BASIONYM_ICON);
	public static final Image MOVE = TaxEditorPlugin.getDefault().getImage(
			ITaxEditorConstants.MOVE_ICON);

	/**
	 * ************ TRANSFORMATIONS ************
	 */
	public static final String ADD_GROUP_BASIONYM = "add_group_basionym";
	public static final String REMOVE_GROUP_BASIONYM = "remove_group_basionym";

	/**
	 * ************ MENU ACTIONS ************
	 */
	public Action CHANGE_TAXON_TO_SYNONYM_ACTION;

	private static final String EMPTY_NAME_PROMPT = "Click to add name";

//	private NameViewer nameViewer;
//	private NameViewer textViewer;

	/**
	 * Used to turn parser on and off.
	 * 
	 * @see activateParser
	 * @see deactivateParser
	 */
	private boolean isUseParser = false;

	protected boolean isParsing;

	private NameViewer nameViewer;

	/**
	 * The constructor for a DescriptionElementComposite. Takes a parent Composite on which to
	 * create itself, and an IManagedForm for Composite life cycle methods, i.e.
	 * drawing borders, creating other Composites, creating line wrap support,
	 * etc.
	 * 
	 * @param parent
	 * @param managedForm
	 */
	public NameComposite(Composite parent, IManagedForm managedForm, String compositeType, TaxonBase data) {
		super(parent, managedForm);

		createNameViewer();
		createBorderSupport();
		createLineWrapSupport();
		
		setData(data);
		
		createNameListener(data);
		
		transform(compositeType);
		
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		createParser();
		
		setFocus();
	}

	/**
	 * Listens for changes to this name's <code>fullTitleCache</code>.
	 * 
	 * @param data
	 */
	private void createNameListener(TaxonBase taxonBase) {
		if (taxonBase.getName() == null) {
			return;
		}
		
		final TaxonNameBase name = taxonBase.getName();
		
		final PropertyChangeListener listener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				
				if (isParsing) {
					return;
				}
				
				deactivateParser();
				((NameViewer) getTextViewer()).setText(name.getFullTitleCache());
				activateParser();
			}
		};
		
		name.addPropertyChangeListener("fullTitleCache", listener);
		name.addPropertyChangeListener("nomenclaturalMicroReference", listener);

		name.addPropertyChangeListener(ITaxEditorConstants.REFRESH_NAMEVIEWER, listener);
		
		StrictReferenceBase reference = (StrictReferenceBase) name.getNomenclaturalReference();
		if (reference != null) {
			reference.addPropertyChangeListener("titleCache", listener);
		}
		
		this.addDisposeListener(new DisposeListener() {

			public void widgetDisposed(DisposeEvent e) {
//				name.removePropertyChangeListener(listener);
			}
		});
	}

	/**
	 * All cosmetic - non-data-related, i.e. icons, fonts, etc. -
	 * transformations take place in this method.
	 * 
	 * @param transformation
	 */
	public void transform(String transformation) {

		if (transformation.equals(ADD_GROUP_BASIONYM)) {
			if (compositeType.equals(HOMOTYPIC_SYNONYM)) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			}
		}
		
		if (transformation.equals(REMOVE_GROUP_BASIONYM)) {
			if (compositeType.equals(HOMOTYPIC_SYNONYM)) {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
		}

		if (transformation.equals(ACCEPTED_TAXON)) {
			setDraggable(false);
			setIcon(ACCEPTED_ICON);
			setFont(ACCEPTED_FONT);
			setIndent(ACCEPTED_INDENT);

			createAcceptedMenu();

			compositeType = ACCEPTED_TAXON;
		}

		if (transformation.equals(HOMOTYPIC_SYNONYM)
				|| transformation.equals(HETEROTYPIC_SYNONYM)) {
			setDraggable(true);
			setFont(SYNONYM_FONT);
			setIndent(SYNONYM_INDENT);

			createSynonymMenu();
		}

		if (transformation.equals(HOMOTYPIC_SYNONYM)) {
			if (!(getData() instanceof Synonym)) {
				return;
			}
			Synonym synonym = (Synonym) getData();
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			}
			compositeType = HOMOTYPIC_SYNONYM;
		}

		if (transformation.equals(HETEROTYPIC_SYNONYM)) {
			if (!(getData() instanceof Synonym)) {
				return;
			}
			Synonym synonym = (Synonym) getData();
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
			compositeType = HETEROTYPIC_SYNONYM;
		}

		if (transformation.equals(MISAPPLIED_NAME)) {

			setDraggable(true);
			setIcon(MISAPPLIEDNAME_ICON);
			setFont(MISAPPLIEDNAME_FONT);
			setIndent(MISAPPLIEDNAME_INDENT);

			createMisappliedNameMenu();

			compositeType = MISAPPLIED_NAME;
		}

		setDirty(true);
		managedForm.getForm().layout();
	}

	/**
	 * Override of Composite.setData() which passes data along to
	 * <code>NameComposite</code>'s <code>TextViewer</code> where appropriate.
	 * 
	 * @see org.eclipse.swt.widgets.Widget#setData(java.lang.Object)
	 */
	public void setData(Object data) {
		super.setData(data);
		
		Assert.isTrue((data instanceof TaxonBase), 
				"NameComposite's data field must contain a TaxonBase object");
		
		String text = CdmUtil.getDisplayNameAndRef((TaxonBase) data);
		getTextViewer().getTextWidget().setText(text);

		if (getTextViewer() instanceof NameViewer) {
			((NameViewer) getTextViewer()).setCursorToEOL();

			if (((TaxonBase) data).getName() != null) {
				boolean hasProblem = ((TaxonBase) data).getName()
						.getHasProblem();
				((NameViewer) getTextViewer()).setShowError(hasProblem);
			}
		}
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.EditorGroupedComposite#setSelection()
	 */
	protected void setSelection() {
		if (getData() instanceof TaxonBase) {
			
			Object selectionInput = null;
			
			if (compositeType.equals(MISAPPLIED_NAME)) {
				
				// A misapplied name's property sheet must display both name and sec. ref.
				selectionInput = getData();
			} else {
				
				// All other composite types display only their name in property sheet
				selectionInput = ((TaxonBase) getData()).getName();
			}
			
			// Send input to the property sheet via the managed form
			managedForm.setInput(selectionInput);
		} else {
			super.setSelection();
		}
	}

	private void createParser() {
		((NameViewer) getTextViewer()).setParseListener(new ParseListener() {

			@Override
			public void parse(String text) {

				isParsing = true;
				
				if (!isUseParser) {
					return;
				}
				
				TaxonBase taxonBase = (TaxonBase) getData();
				NonViralName nonViralName = (NonViralName) taxonBase.getName();
				
				if (nonViralName != null) {
					CdmUtil.parseFullReference(nonViralName, text, null, true);
					((NameViewer) getTextViewer()).setShowError(nonViralName.getHasProblem());
				}

				setDirty(true);
				
				isParsing = false;
			}
		});
		activateParser();
	}

	public void activateParser() {
		isUseParser = true;
	}
	
	public void deactivateParser() {
		isUseParser = false;
	}
	
	private NameViewer createNameViewer() {
//		nameViewer = new NameViewer(this);
//		nameViewer.setLineBreakListener(new LineBreakListener() {
		
		setTextViewer(new NameViewer(this));
		getTextViewer().setLineBreakListener(new LineBreakListener() {

			@Override
			public void handleSplitText(String text) {

				Composite parent = getParent();
				final Composite grandParent = parent.getParent();

				new CreateNewHeterotypicCompositeAction(text, managedForm)
						.run();

				// setDirty(true);
			}

		});

//		nameViewer.getTextWidget().addFocusListener(new FocusAdapter() {
		getTextViewer().getTextWidget().addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setFocus();
			}
		});

		// createLineWrapSupport(nameViewer);
//
//		this.textViewer = nameViewer;
//		nameViewer.getTextWidget().setBackground(TaxEditorPlugin.getDefault().
//				getColor(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR));
		getTextViewer().getTextWidget().setBackground(TaxEditorPlugin.getDefault().
				getColor(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR));		
		
		return (NameViewer) getTextViewer();
	}

	/**
	 * @param nameComposite
	 */
	private void createSynonymMenu() {

		if (!(getData() instanceof Synonym)) {
			return;
		}

		Synonym synonym = (Synonym) getData();
		ContextMenu contextMenu = createContextMenu();

		Action misappliedNameAction = new MoveCompositeToMisappliedCompositeAction(
				this, managedForm);
		contextMenu.addAction(misappliedNameAction);

		Action deleteSynonymAction = new DeleteSynonymCompositeAction(this,
				getTaxon());
		contextMenu.addAction(deleteSynonymAction);

		contextMenu.addSeparator();

		final Action addBasionymAction = new AddBasionymCompositeAction(this);
		contextMenu.addAction(addBasionymAction);

		final Action removeBasionymAction = new RemoveBasionymCompositeAction(
				this);
		contextMenu.addAction(removeBasionymAction);

		addBasionymAction
				.addPropertyChangeListener(new IPropertyChangeListener() {
					public void propertyChange(
							org.eclipse.jface.util.PropertyChangeEvent event) {
						if (event.getProperty().equals(
								ITaxEditorConstants.BASIONYM)) {
							removeBasionymAction.setEnabled(true);
							transform(NameComposite.ADD_GROUP_BASIONYM);
						}
					}
				});

		removeBasionymAction
				.addPropertyChangeListener(new IPropertyChangeListener() {
					public void propertyChange(
							org.eclipse.jface.util.PropertyChangeEvent event) {
						if (event.getProperty().equals(
								ITaxEditorConstants.BASIONYM)) {
							addBasionymAction.setEnabled(true);
							transform(NameComposite.REMOVE_GROUP_BASIONYM);
						}
					}
				});

		contextMenu.addSeparator();

		Action changeToThisTaxon = new ChangeSynonymToTaxonUiAction(synonym,
				getTaxon());
		contextMenu.addAction(changeToThisTaxon);

		Action changeToNewAccepted = new ChangeCompositeToNewTaxonAction(this,
				getTaxon());
		contextMenu.addAction(changeToNewAccepted);
	}

	/**
	 * @param nameComposite
	 */
	private void createAcceptedMenu() {

		ContextMenu contextMenu = createContextMenu();

		// TODO Make action "Create autonym and subspecies"

		Action changeTaxonAction = new ChangeTaxonToSynonymAction(getTaxon());
		contextMenu.addAction(changeTaxonAction);

		Action moveTaxonAction = new MoveTaxonDialogAction(getTaxon());
		contextMenu.addAction(moveTaxonAction);
	}

	/**
	 * @param nameComposite
	 */
	private void createMisappliedNameMenu() {

		ContextMenu contextMenu = createContextMenu();

		Action deleteMisappliedNameAction = new DeleteMisappliedNameCompositeAction(
				this, getTaxon());
		contextMenu.addAction(deleteMisappliedNameAction);
	}

	public boolean setParent(Composite parent) {

		if (super.setParent(parent)) {

			// Has this been moved to the misapplied names group?
			if (parent.getData(ITaxEditorConstants.MISAPPLIED_NAME) != null) {
				new ChangeCompositeToMisappliedNameAction(this, managedForm)
						.run();
			}

			// Has this been moved to a HomotypicalGroup?
			if (parent.getData() instanceof HomotypicalGroup) {
				new AdaptCompositeToGroupAction(this,
						(EditorGroupComposite) parent).run();
			}
			return true;

		}
		return false;
	}
	
	public void setText(String text) {
		Assert.isNotNull(getTextViewer(), 
				"Cannot set text for a TextViewer that has not yet been initialized.");
		Assert.isNotNull(getTextViewer().getDocument(), 
				"Cannot set text for a TextViewer whose Document has not yet been initialized.");
		getTextViewer().getDocument().set(text);
	}
	
	public NameViewer getTextViewer() {
		return this.nameViewer;
	}
	
	public void setTextViewer(NameViewer textViewer) {
		this.nameViewer = textViewer;
	}
}