/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ILineTracker;
import org.eclipse.jface.text.IUndoManager;
import org.eclipse.jface.text.IUndoManagerExtension;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.TextViewerUndoManager;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.AnnotationPainter;
import org.eclipse.jface.text.source.IAnnotationAccess;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.operations.OperationHistoryActionHandler;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import eu.etaxonomy.taxeditor.editor.AnnotationMarkerAccess;
import eu.etaxonomy.taxeditor.editor.ErrorAnnotation;
import eu.etaxonomy.taxeditor.editor.LineBreakListener;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.editor.WarningAnnotation;


/**
 * SourceViewer implementation called by NameComposite.
 *  
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class NameViewer extends SourceViewer {
	private static final Logger logger = Logger
			.getLogger(NameViewer.class);

	public NameViewer(Composite parent) {
		super(parent, null, SWT.WRAP | SWT.MULTI | SWT.RESIZE);
							
		// Set name viewer's text to name cache
		AnnotationModel model = new AnnotationModel();
		NameViewerDocument document = new NameViewerDocument("");
		this.setDocument(document, model);
//		model.connect(document);

		// Lay out StyledText
		StyledText textWidget = this.getTextWidget();
		textWidget.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		
		// Annotations section
		IAnnotationAccess fAnnotationAccess = new AnnotationMarkerAccess();
		
		// To paint the annotations
		AnnotationPainter annotationPainter = new AnnotationPainter(this, fAnnotationAccess);

		// Add ability to paint red squigglies
		annotationPainter.addAnnotationType(ErrorAnnotation.ERROR_TYPE);
		annotationPainter.setAnnotationTypeColor(ErrorAnnotation.ERROR_TYPE, 
				new Color(Display.getDefault(), ErrorAnnotation.ERROR_RGB));

		// Add ability to paint yellow squigglies
		annotationPainter.addAnnotationType(WarningAnnotation.WARNING_TYPE);
		annotationPainter.setAnnotationTypeColor(WarningAnnotation.WARNING_TYPE, 
				new Color(Display.getDefault(), WarningAnnotation.WARNING_RGB));		
		
		this.addPainter(annotationPainter);
		
//		DocumentUndoManagerRegistry.connect(this.getDocument());
//		IDocumentUndoManager docUndoManager = DocumentUndoManagerRegistry.getDocumentUndoManager(this.getDocument());
		
		/**
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.UNDO.getId(), new UndoActionHandler(getSite(), undoContext));
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.REDO.getId(), new RedoActionHandler(getSite(), undoContext));		
		**/
		

	}
	
	/**
	 * NOTE: possible solution to problem of one-lined red squigglies:
	 * AnnotationPainter uses an internal SquigglesStrategy which draws a 
	 * polyline consisting of diagonal lines; this assumes one line;
	 * re-program to catch more than one lines, using 
	 * this.getTextWidget().getLocationAtOffset to find linecount.
	 * 
	 * @param hasProblem
	 */
	public void setShowError(boolean hasProblem) {
		
		String text = this.getTextWidget().getText();
		
//		logger.warn(this.getTextWidget().getBaseline(0));
//		logger.warn(this.getTextWidget().getLocationAtOffset(60));
//		
//		if (text.length() > 0) {
//			logger.warn(this.getTextWidget().getTextBounds(0, text.length()));
//		}
		
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next(); 
			if (annotation.getType().equals(ErrorAnnotation.ERROR_TYPE))
				annotation.markDeleted(true);
				this.getAnnotationModel().removeAnnotation(annotation);
		}
		
		if (hasProblem && text.length() > 0) {
			this.getAnnotationModel().addAnnotation(
					new ErrorAnnotation(0, text), 
					new Position(0, text.length()));
//			this.getAnnotationModel().addAnnotation(
//					new ErrorAnnotation(0, text), 
//					new Position(0, 40));
//			this.getAnnotationModel().addAnnotation(
//					new ErrorAnnotation(0, text), 
//					new Position(39, text.length()));				
			
		}
	}
	
	public void setLineBreakListener(LineBreakListener lineBreakListener) {
		this.getTextWidget().addVerifyListener(lineBreakListener);
	}
	
	public void setParseListener(ParseListener parseListener) {
		this.getTextWidget().addModifyListener(parseListener);
	}

	public void setText(String text) {
		if (text == null) {
			text = "";
		}
		try {
			Assert.isNotNull(text);
			// TODO figure out why getTextWidget() returns null!
			if (this.getTextWidget() == null) {
				return;
			}
			Assert.isNotNull(this.getTextWidget());
			this.getTextWidget().setText(text);
		} catch (RuntimeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setCursorToEOL() {
		this.getTextWidget().setCaretOffset(getTextWidget().getText().length());
	}
	
	class NameViewerDocument extends Document {
		public NameViewerDocument(String string) {
			super(string);
		}

		public ILineTracker getTracker() {
			return super.getTracker();
		}	
	}

	public void createUndoSupport(IEditorSite editorSite) {

		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		
//		IUndoContext workbenchUndoContext = UiUtil.getWorkbenchUndoContext();
		
		IUndoContext workbenchUndoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
		OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, workbenchUndoContext);
//		undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
		undoAction.setActionDefinitionId(ActionFactory.UNDO.getId());
		
		// Create the redo action.
		OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, workbenchUndoContext);
//		redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);
		undoAction.setActionDefinitionId(ActionFactory.REDO.getId());

		IActionBars actionBars = editorSite.getActionBars();
		if (actionBars != null) {
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
			actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
			actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
			
		}		
		
//		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
//		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
	}
	
	public void createUndoSupport_(IEditorSite editorSite) {
		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		IUndoContext undoContext;
		if (undoManager instanceof IUndoManagerExtension) {
			undoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
			OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(undoAction, IAbstractTextEditorHelpContextIds.UNDO_ACTION);
			undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
			
			// Create the redo action.
			OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(redoAction, IAbstractTextEditorHelpContextIds.REDO_ACTION);
			redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);


			IActionBars actionBars = editorSite.getActionBars();
			if (actionBars != null) {
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
				
			}
		}
	}
	
//	public boolean isMultiLine(Annotation annotation) {
//	    return true;
//	}
}
