/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.MessageDialog;
import org.springframework.transaction.TransactionStatus;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.CdDvd;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.dao.hibernate.common.IdentifiableDaoBase;
import eu.etaxonomy.cdm.strategy.parser.INonViralNameParser;
import eu.etaxonomy.cdm.strategy.parser.NonViralNameParserImpl;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;

/**
 * @author p.ciardelli
 * @created 26.05.2008
 * @version 1.0
 */
/**
 * @author p.ciardelli
 * @created 17.09.2008
 * @version 1.0
 */
public class CdmUtil {
	private static final Logger logger = Logger.getLogger(CdmUtil.class);

	private static INonViralNameParser nonViralNameParser;
	private static TermVocabulary<Feature> features;
	private static SortedSet<Rank> ranks;

	/**
	 * Clears all objects retrieved from <code>cdmlib</code> 
	 * to force them to re-initialize the next time they
	 * are called.
	 */
	public static void clearLibraryObjects() {
		features = null;
		ranks = null;
		nonViralNameParser = null;
	}
	
	/**
	 * Checks whether synonym's name is the basionym for ALL names 
	 * 	in its group.
	 * 
	 * @param synonym
	 * 
	 * @return
	 */
	public static boolean isSynonymGroupBasionym(Synonym synonym) {
		
		TaxonNameBase synonymName = synonym.getName();
		return isNameGroupBasionym(synonymName);
	}
	
	/**
	 * Checks whether name is the basionym for ALL names 
	 * 	in its group.
	 * @param name
	 * 
	 * @return
	 */	
	public static boolean isNameGroupBasionym(TaxonNameBase name) {
		if (name == null) {
			return false;
		}
		
		HomotypicalGroup homotypicalGroup = name.getHomotypicalGroup();
		if (homotypicalGroup == null) {
			return false;
		}		
		
		Set<TaxonNameBase> typifiedNames = homotypicalGroup.getTypifiedNames();
		
		// Check whether there are any other names in the group
		if (typifiedNames.size() == 1) {
			return false;
		}
		
		boolean isBasionymToAll = true;
				
		for (TaxonNameBase taxonName : typifiedNames) {
			if (!taxonName.equals(name)) {
				if (!isNameBasionymOf(name, taxonName)) {
					return false;
				}
			}
		}
		return true;		
	}
	
	/**
	 * Checks whether a basionym relationship exists between fromName and toName.
	 * 
	 * @param fromName
	 * @param toName
	 * @return
	 */
	public static boolean isNameBasionymOf(TaxonNameBase fromName, TaxonNameBase toName) {
		Set<NameRelationship> relations = toName.getRelationsToThisName();
		for (NameRelationship relation : relations) {
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getFromName().equals(fromName)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Creates a basionym relationship between basionymName and
	 * 	each name in its homotypic group.
	 * 
	 * @param basionymName
	 */
	public static void setGroupBasionym(TaxonNameBase basionymName) {
		HomotypicalGroup homotypicalGroup = basionymName.getHomotypicalGroup();
		if (homotypicalGroup == null) {
			return;
		}
		for (TaxonNameBase name : homotypicalGroup.getTypifiedNames()) {
			if (!name.equals(basionymName)) {
				
				// First check whether the relationship already exists
				if (!isNameBasionymOf(basionymName, name)) {
					
					// Then create it
					name.addRelationshipFromName(basionymName, 
							NameRelationshipType.BASIONYM(), null);
				}
			}
		}
	}

	/**
	 * Removes all basionym relationships between basionymName and
	 * 	the names in its homotypic group.
	 * 
	 * @param basionymName
	 */
	public static void removeGroupBasionym(TaxonNameBase basionymName) {
		HomotypicalGroup homotypicalGroup = basionymName.getHomotypicalGroup();
		Set<NameRelationship> relations = basionymName.getRelationsFromThisName();
		Set<NameRelationship> removeRelations = new HashSet<NameRelationship>();
		
		for (NameRelationship relation : relations) {
			
			// If this is a basionym relation, and toName is in the homotypical group,
			//	remove the relationship.
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getToName().getHomotypicalGroup().equals(homotypicalGroup)) {
				removeRelations.add(relation);
			}
		}
		
		// Removing relations from a set through which we are iterating causes a 
		//	ConcurrentModificationException. Therefore, we delete the targeted
		//	relations in a second step.
		for (NameRelationship relation : removeRelations) {
			basionymName.removeNameRelationship(relation);
		}
	}
	
	/**
	 * Checks whether name belongs to the same homotypic group as taxon's name.
	 * 
	 * @param name
	 * @param taxon
	 * @return
	 */
	public static boolean isNameHomotypic(TaxonNameBase name, Taxon taxon) {
		TaxonNameBase taxonName = taxon.getName();
		if (taxonName == null || name == null) {
			return false;
		}
		HomotypicalGroup homotypicGroup = taxonName.getHomotypicalGroup();
		if (homotypicGroup == null) {
			return false;
		}
		if (homotypicGroup.equals(name.getHomotypicalGroup())) {
			return true;
		}
		return false;
	}

	/**
	 * Returns whatever is currently considered the display name for Taxon
	 * objects. Currently titleCache.
	 * 
	 * @param taxon
	 * @return
	 */
	public static String getDisplayName(TaxonBase taxonBase) {
		TaxonNameBase name = taxonBase.getName();
		return getDisplayName(name);
	}

	/**
	 * @param name
	 * @return
	 */
	public static String getDisplayName(TaxonNameBase name) {
		if (name != null) {
			return name.getTitleCache();			
		}
		return "";
	}
	
	public static String getDisplayNameAndRef(TaxonBase taxonBase) {
		TaxonNameBase name = taxonBase.getName();
		if (name != null) {
			return name.getFullTitleCache();
		}
		return "";
	}

	/**
	 * @return
	 */
	private static INonViralNameParser getNonViralNameParser() {
		if (nonViralNameParser == null) {
			nonViralNameParser = NonViralNameParserImpl.NewInstance();
		}
		return nonViralNameParser;
	}

	/**
	 * Takes a raw string, returns a brand spanking new name object.
	 * <p>
	 * If no nomenclatural code is specified, the code specified in user preferences
	 * is used, i.e. if the preferred code is ICBN, a <code>BotanicalName</code> is returned; 
	 * if it's ICZN, a <code>ZoologicalName</code> is returned.
	 * </p>
	 * 
	 * @param fullReference
	 * @param nomCode
	 * @param rank
	 * @return
	 */
	public static TaxonNameBase parseFullReference(String fullReference,
			NomenclaturalCode nomCode, Rank rank) {
		
		if (nomCode == null) {
			nomCode = UiUtil.getPreferredNomenclaturalCode();
		}
		
		TaxonNameBase name = getNonViralNameParser().parseFullReference(fullReference,
				nomCode, rank);

		if (name.hasProblem()) {
			name.setFullTitleCache(fullReference);
		}
		
		return name;
	}

	/**
	 * Takes an existing name object and a string, returns the same name
	 * object with the fields parsed from the string.
	 * 
	 * @param nameToBeFilled
	 * @param fullReference
	 * @param rank
	 * @param makeEmpty
	 */
	public static void parseFullReference(NonViralName nameToBeFilled, String fullReference, Rank rank, boolean makeEmpty) {
		getNonViralNameParser().parseFullReference(nameToBeFilled, fullReference, rank, makeEmpty);
		
		if (nameToBeFilled.hasProblem()) {
			nameToBeFilled.setFullTitleCache(fullReference);
		}
	}
	
	/**
	 * @param nameToBeFilled
	 * @param fullNameString
	 * @param rank
	 * @param makeEmpty
	 */
	public static void parseFullName(NonViralName nameToBeFilled,
			String fullNameString, Rank rank, boolean makeEmpty) {
		getNonViralNameParser().parseFullName(nameToBeFilled, fullNameString,
				rank, makeEmpty);
	}


	
	/**
	 * @return
	 */
	public static ReferenceBase getSessionDefaultSec() {
		return null;
//		return TaxEditorPlugin.getDefault().getSec();
	}

	/**
	 * @param oldTaxon
	 * @param newAcceptedTaxon
	 * @param synonymType
	 * @param citation
	 * @param citationMicroReference
	 */
	public static void makeTaxonSynonym(Taxon oldTaxon, Taxon newAcceptedTaxon,
			SynonymRelationshipType synonymType, ReferenceBase citation,
			String citationMicroReference) {
		ITaxonService taxonService = getTaxonService();
		taxonService.makeTaxonSynonym(oldTaxon, newAcceptedTaxon, synonymType,
				citation, citationMicroReference);
	}

	/**
	 * @param searchText
	 * @return
	 */
	public static Set<TaxonNameBase> getNameByName(String searchText) {
		Set<TaxonNameBase> resultsSet = new HashSet<TaxonNameBase>();
		resultsSet.addAll(getNameService()
				.getNamesByName(searchText.replace("*", "%")));
		return resultsSet;
	}
	
	public static INameService getNameService() {
		return TaxEditorPlugin.getDefault().getNameService();
	}
	
	public static ITaxonService getTaxonService() {
		return TaxEditorPlugin.getDefault().getTaxonService();
	}

	public static IDescriptionService getDescriptionService() {
		return TaxEditorPlugin.getDefault().getDescriptionService();
	}

	public static IReferenceService getReferenceService() {
		return TaxEditorPlugin.getDefault().getReferenceService();
	}
	
	public static TermVocabulary<Feature> getFeatures() {
		if (features == null) {
			features = CdmUtil.getDescriptionService().
					getDefaultFeatureVocabulary();
		}
		return features;
	}
	
	public static SortedSet<Rank> getRanks() {
		if (ranks == null) {
//			TransactionStatus tx = startReadOnlyTransaction();
			OrderedTermVocabulary<Rank> rankVocabulary = TaxEditorPlugin.getDefault().getRankVocabulary();
			ranks = rankVocabulary.getOrderedTerms(null);
//			commitTransaction(tx);
		}
		return ranks;
	}
	
	/**
	 * @param searchText
	 * @param taxon
	 * @return
	 */
	public static Set<TaxonNameBase> getNameByNameForTaxonContext(String searchText, Taxon taxon) {
		Set<TaxonNameBase> resultsSet = new HashSet<TaxonNameBase>();
		resultsSet.addAll(getNameService().getNamesByName
				(searchText.replace("*", "%"), taxon));
		return resultsSet;
	}
	
	public static TransactionStatus startTransaction() {
		return TaxEditorPlugin.getDefault().getCdmApp().startTransaction();
	}
	
	public static TransactionStatus startReadOnlyTransaction() {
		return TaxEditorPlugin.getDefault().getCdmApp().startTransaction(true);
	}
	
	public static void commitTransaction(TransactionStatus tx) {
		TaxEditorPlugin.getDefault().getCdmApp().commitTransaction(tx);
	}
	
	public static void getNomenclaturalReference(TaxonNameBase name) {
//    	INomenclaturalReference nomenclaturalReference = (INomenclaturalReference) name.getNomenclaturalReference();
//    	if (nomenclaturalReference == null) {
//    		return "";
//    	}        	
//    	String microReference = name.getNomenclaturalMicroReference();
//		return CdmUtils.Nz(nomenclaturalReference.getNomenclaturalCitation(microReference));
	}
	
	/**
	 * Checks whether a String is either a valid year 
	 *  or two valid years
	 * with the format "XXXX-XXXX". 
	 *
	 * @see #getValidYear(String yearStr)
	 * @param refYear
	 * @return
	 * @throws NumberFormatException
	 */
	public static TimePeriod getDatePublished(String refYear) throws NumberFormatException {
		
		if (refYear == null || ("").equals(refYear)){
			return null;
		}
		
		TimePeriod datePublished = TimePeriod.NewInstance();
		
		// In case format is "xxxx-xxxx"
		String[] years = refYear.split("-");

		// Unlikely case of "xxx-xx-xxx..."
		if (years.length > 2) {
			throw new NumberFormatException();
		}
		
		// Set startYear
		datePublished.setStartYear(getValidYear(years[0]));

		// Format is "xxx-xxxx"
		if (years.length == 2) {
			datePublished.setEndYear(getValidYear(years[1]));
		}
		
		return datePublished;
	}
	
	
	/**
	 * Checks whether a <code>String</code> is a valid year between
	 * 1750 and 2030. Throws a <code>NumberFormatException</code> if not.
	 * 
	 * @param yearStr
	 * @return
	 * @throws NumberFormatException
	 */
	public static Integer getValidYear(String yearStr) throws NumberFormatException {
		
		Integer yearInt = null;
		
		// Try casting string - don't catch number format exception
		try {
			yearInt = new Integer(yearStr);
		} catch (ClassCastException e) {
			throw new NumberFormatException();
		}
		
		// Is year in valid range?
		if (yearInt < 1750 || yearInt > 2030) {
			throw new NumberFormatException();
		}
		
		return yearInt;
	}

	/**
	 * Searches for references by string. "%" is used as a wildcard.
	 * 
	 * @param text
	 * @return 
	 */
	public static List getReferencesByTitle(String reference) {
		
		reference = reference.replace("*", "%");
		List resultsList = null;
		try {
			resultsList = getReferenceService().getReferencesByTitle(reference);
		} catch (RuntimeException e) {
			MessageDialog.openError(UiUtil.getShell(), "Search reference error", 
					"Reference search returned an error. This could be a Hibernate concurrency problem. " +
					"Please try saving your work, then searching again.");
			e.printStackTrace();
		}
		return resultsList;
	}
}
