/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;

/**
 * @author p.ciardelli
 * @created 10.11.2008
 * @version 1.0
 */
public class TimePeriodPropertySource implements IPropertySource {
	private static final Logger logger = Logger
			.getLogger(TimePeriodPropertySource.class);
	
	private TimePeriod timePeriod;
	
	// Property unique keys
	public static final String P_ID_STARTYEAR = "P_ID_STARTYEAR";
	public static final String P_ID_STARTMONTH = "P_ID_STARTMONTH";
	public static final String P_ID_STARTDAY = "P_ID_STARTDAY";
	public static final String P_ID_ENDYEAR = "P_ID_ENDYEAR";
	public static final String P_ID_ENDMONTH = "P_ID_ENDMONTH";
	public static final String P_ID_ENDDAY = "P_ID_ENDDAY";
		
	// Property display keys
	public static final String P_STARTYEAR = "00:Year (Start)";
	public static final String P_STARTMONTH = "01:Month (Start)";
	public static final String P_STARTDAY = "02:Day (Start)";
	public static final String P_ENDYEAR = "03:Year (End)";
	public static final String P_ENDMONTH = "04:Month (End)";
	public static final String P_ENDDAY = "05:Day (End)";	

	protected static final String[] TOP_LEVEL_PROPERTIES = 
		new String[] {P_ID_STARTYEAR, P_ID_STARTMONTH, P_ID_STARTDAY, 
						P_ID_ENDYEAR, P_ID_ENDMONTH, P_ID_ENDDAY};
			
	private static final String[] P_MONTH_MENU = new String[] {"", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
	
	public TimePeriodPropertySource(TimePeriod timePeriod) {
		super();
		
		if (timePeriod == null) {
			timePeriod = TimePeriod.NewInstance();
		}
		this.timePeriod = timePeriod;

		// Add property sheet descriptors
		for (String key : TOP_LEVEL_PROPERTIES) {
			addDescriptor(key);
		}
	}
		
    //static date formatter
    private static final DateFormat formatter = new SimpleDateFormat(
            "EEEE, MMMM d, yyyy"); //$NON-NLS-1$

    private Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
    
    private void addDescriptor(String id) {
        
        if (id.equals(P_ID_STARTYEAR)) {
        	PropertyDescriptor descriptor = new TextPropertyDescriptor(P_ID_STARTYEAR, P_STARTYEAR);
        	descriptor.setValidator(new YearValidator());
			descriptors.addElement(descriptor);
        }

        if (id.equals(P_ID_STARTMONTH)) {
        	descriptors.addElement(new ComboBoxPropertyDescriptor(
        			P_ID_STARTMONTH, P_STARTMONTH, P_MONTH_MENU));
        }

        if (id.equals(P_ID_STARTDAY)) {
        	PropertyDescriptor descriptor = new TextPropertyDescriptor(P_ID_STARTDAY, P_STARTDAY);
        	descriptor.setValidator(new DayValidator());
			descriptors.addElement(descriptor);
        }
        
        if (id.equals(P_ID_ENDYEAR)) {
        	PropertyDescriptor descriptor = new TextPropertyDescriptor(P_ID_ENDYEAR, P_ENDYEAR);
        	descriptor.setValidator(new YearValidator());
			descriptors.addElement(descriptor);
        }

        if (id.equals(P_ID_ENDMONTH)) {
        	descriptors.addElement(new ComboBoxPropertyDescriptor(
        			P_ID_ENDMONTH, P_ENDMONTH, P_MONTH_MENU));
        }
        
        if (id.equals(P_ID_ENDDAY)) {
        	PropertyDescriptor descriptor = new TextPropertyDescriptor(P_ID_ENDDAY, P_ENDDAY);
        	descriptor.setValidator(new DayValidator());
			descriptors.addElement(descriptor);
        }
    }

    /* (non-Javadoc)
     * Method declared on IPropertySource
     */
    public Object getEditableValue() {
        return "";
    }

    /* (non-Javadoc)
     * Method declared on IPropertySource
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
    }

    public Object getPropertyValue(Object id) {
    	
    	if (id.equals(P_ID_STARTYEAR)) {
    		Integer startYear = timePeriod.getStartYear();
    		return (startYear == null) ? "" : String.valueOf(startYear);
    	}

    	if (id.equals(P_ID_STARTMONTH)) {
    		Integer startMonth = timePeriod.getStartMonth();
    		return (startMonth == null) ? 0 : startMonth;
    	}

    	if (id.equals(P_ID_STARTDAY)) {
    		Integer startDay = timePeriod.getStartDay();    		
    		return (startDay == null) ? "" : String.valueOf(startDay);
    	}
    	
    	if (id.equals(P_ID_ENDYEAR)) {
    		Integer endYear = timePeriod.getEndYear();
    		return (endYear == null) ? "" : String.valueOf(endYear);
    	}

    	if (id.equals(P_ID_ENDMONTH)) {
    		Integer endMonth = timePeriod.getEndMonth();
    		return (endMonth == null) ? 0 : endMonth;
    	}

    	if (id.equals(P_ID_ENDDAY)) {
    		Integer endDay = timePeriod.getEndDay();
    		return (endDay == null) ? "" : String.valueOf(endDay);
    	}
    	
        return "";
    }

    /* (non-Javadoc)
     * Method declared on IPropertySource
     */
    public boolean isPropertySet(Object property) {
    	return false;
    }

    /* (non-Javadoc)
     * Method declared on IPropertySource
     */
    public void resetPropertyValue(Object property) {}

    public void setPropertyValue(Object id, Object value) {

	   	if (id.equals(P_ID_STARTYEAR)) {
			timePeriod.setStartYear(castToInteger(value));
		}
	
		if (id.equals(P_ID_STARTMONTH)) {
			timePeriod.setStartMonth(castToInteger(value));
		}
	
		if (id.equals(P_ID_STARTDAY)) {
			timePeriod.setStartDay(castToInteger(value));
		}
		
		if (id.equals(P_ID_ENDYEAR)) {
			timePeriod.setEndYear(castToInteger(value));
		}
	
		if (id.equals(P_ID_ENDMONTH)) {
			timePeriod.setEndMonth(castToInteger(value));
		}
	
		if (id.equals(P_ID_ENDDAY)) {
			timePeriod.setEndDay(castToInteger(value));
		}
    	
    	propertyChangeSupport.firePropertyChange(ITaxEditorConstants.PROPERTY_SHEET_CHANGE, null, timePeriod);
    }

    private Integer castToInteger(Object value) {
    	
    	// Dropdown lists return an Integer index
    	if (value instanceof Integer) {
    		
    		// First entry in dropdown is empty
    		if (((Integer) value).equals(0)) {
    			return null;
    		} else {
    			return (Integer) value; 
    		}
    	}
    	 
    	Integer integer = null;
		try {
			integer = new Integer((String) value);
			
		} catch (ClassCastException e) {
		} catch (NumberFormatException e) {
		}
    	return integer;
    }
    
    /**
     * The value as displayed in the Property Sheet.
     * @return java.lang.String
     */
    public String toString() {
    	return "";
    }	

	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(
			PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}
}
